import { Plugin, ButtonView } from 'ckeditor5';

import ckeditor5Icon from '../theme/icons/ckeditor.svg';

import { asBlob } from 'html-docx-js-typescript';
import { saveAs } from 'file-saver';

import Temml from 'temml';

export default class Import extends Plugin {
	public static get pluginName() {
		return 'Import' as const;
	}

	public init(): void {
		const editor = this.editor;
		const t = editor.t;
		const model = editor.model;

		// Add the "importButton" to feature components.
		editor.ui.componentFactory.add( 'importButton', locale => {
			const view = new ButtonView( locale );

			view.set( {
				label: t( 'Import' ),
				icon: ckeditor5Icon,
				tooltip: true
			} );

			// POST fetch html from docx
			this.listenTo( view, 'execute', async () => {
				// curl -X 'POST' \
				// 'http://localhost:8080/aspose/word-to-html' \
				// -H 'accept: text/plain' \
				// -H 'Content-Type: application/vnd.openxmlformats-officedocument.wordprocessingml.document' \
				// --data-binary '@stou(207).docx'

				// file accpt docx
				const input = document.createElement('input');
				input.type = 'file';
				input.accept = '.docx';
				input.click();

				input.addEventListener('change', async () => {
					const file = input.files![0];
					const formData = new FormData();
					formData.append("file", file);

					try {
						const response = await fetch("http://localhost:8080/aspose/word-to-html", {
							method: "POST",
							headers: {
								// Do not set `Content-Type` manually when using FormData;
								// the browser will automatically set the correct boundary for multipart/form-data.
							},
							body: file, // directly sending file bytes as the backend expects byte[]
						});

						if (!response.ok) {
							throw new Error(`Failed to upload file: ${response.statusText}`);
						}

						const htmlText = await response.text();
						// console.log("Converted HTML:", htmlText);
						const htmlDoc = new DOMParser().parseFromString(htmlText, "text/html");
						const body = htmlDoc.body;
						this.editor.setData(body.innerHTML);
					} catch (error) {
						console.error("Error uploading file:", error);
						throw error;
					}

					editor.editing.view.focus();
				});
			} );

			return view;
		} );
	}
}
